package devices

import (
	"log"
	"time"
)

type Device interface {
	Status() (bool, error)
	PushPowerButton() error
}

func KeepDeviceInSync(logger *log.Logger, device Device, wantStatus func() (bool, error), exit <-chan bool) {
	for {
		select {
		case <-exit:
			break
		default:
			status, err := device.Status()
			if err != nil {
				logger.Printf("Failed to fetch device status, waiting 1 minute until next attempt: %s", err)
				time.Sleep(1 * time.Minute)
				continue
			}

			want, err := wantStatus()
			if err != nil {
				logger.Printf("Failed to fetch want status, waiting 1 minute until next attempt: %s", err)
				time.Sleep(1 * time.Minute)
				continue
			}

			if status == want {
				// Don't lock up our single-threaded session storage too much
				time.Sleep(2 * time.Second)
				continue
			}

			err = device.PushPowerButton()
			if err != nil {
				logger.Printf("Failed to push power button, waiting 2 minutes until next attempt: %s", err)
				// Cooldown to not overwhelm the device with start/stop requests
				time.Sleep(2 * time.Minute)
				return
			}

			logger.Printf("Pushed power button")
			time.Sleep(2 * time.Minute)
		}
	}
}
